<?php

namespace FiloBlu\Referrals\Controller\Subscriber\Account;


use Magento\Customer\Api\AccountManagementInterface as CustomerAccountManagement;
use Magento\Customer\Model\Session;
use Magento\Customer\Model\Url as CustomerUrl;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Validator\EmailAddress as EmailValidator;
use Magento\Framework\View\Result\PageFactory;
use Magento\Invitation\Model\InvitationProvider;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Message\ManagerInterface;


class Create extends \Magento\Newsletter\Controller\Subscriber\NewAction
{

    /**
     * @var PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var InvitationProvider
     */
    protected $invitationProvider;

    /**
     * @var \FiloBlu\Referrals\Helper\Core
     */
    protected $coreHelper;

    /**
     * @var SubscriberFactory
     */
    protected $subscriberFactory;

    public function __construct(
        Context $context ,
        SubscriberFactory $subscriberFactory ,
        Session $customerSession ,
        StoreManagerInterface $storeManager ,
        CustomerUrl $customerUrl ,
        CustomerAccountManagement $customerAccountManagement ,
        PageFactory $resultPageFactory,
        InvitationProvider $invitationProvider,
        ManagerInterface $messageManager,
        \FiloBlu\Referrals\Helper\Core $coreHelper,
        EmailValidator $emailValidator = null
    )
    {
        $this->resultPageFactory = $resultPageFactory;
        $this->invitationProvider = $invitationProvider;
        parent ::__construct($context , $subscriberFactory , $customerSession , $storeManager , $customerUrl , $customerAccountManagement , $emailValidator);
        $this->coreHelper = $coreHelper;
        $this->subscriberFactory = $subscriberFactory;

    }

    public function execute()
    {

        /** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();

        try {
            $invitation = $this->invitationProvider->get($this->getRequest());

            $customerId = $this->_customerSession->getCustomerId();

            /**
             * TODO: cast customer id quando si usa la newsletter
             */
            if (!$customerId) {

                $couponInvitationType = $this->coreHelper->getSysConfigValue($this->coreHelper::COUPON_FIELD_INVITATION_TYPE);
                if ($couponInvitationType !== 'site') {

                    $subscriber = $this->subscriberFactory->create()->loadByEmail($invitation->getEmail());

                    if (!$subscriber->getSubscriberId()) {
                        $this->subscriberFactory->create()->subscribe($invitation->getEmail());
                        $subscriber = $this->subscriberFactory->create()->loadByEmail($invitation->getEmail());
                    }


                    $invitation->setData('subscriber_id', $subscriber->getSubscriberId());

                    $customerId = $invitation->getCustomerId();
                }

            }

            $invitation->accept($this->_storeManager->getWebsite()->getId(), $customerId);

            $resultRedirect->setPath('home');
            return $resultRedirect;

        } catch (\Exception $e) {
            $this->_customerSession->setCustomerFormData($this->getRequest()->getPostValue());
            $this->messageManager->addException($e, __('We can\'t save this customer.'));
        }

        $resultRedirect->setPath('magento_invitation/account/login', ['_current' => true, '_secure' => true]);
        return $resultRedirect;
    }
}