<?php


namespace FiloBlu\Referrals\Model;


use Magento\Framework\App\ObjectManager;

class Invitation extends \Magento\Invitation\Model\Invitation
{
    const XML_PATH_REFERRAL_EMAIL_TEMPLATE = 'filoblu_referrals/general/templateinvitation';

    /**
     * Send invitation email
     *
     * @return bool
     */
    public function sendInvitationEmail()
    {
        $this->makeSureCanBeSent();
        $store = $this->_storeManager->getStore($this->getStoreId());

        $templateIdentifier = $this->_scopeConfig->getValue(
            self::XML_PATH_REFERRAL_EMAIL_TEMPLATE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
        $from = $this->_scopeConfig->getValue(
            self::XML_PATH_EMAIL_IDENTITY,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );

        $urls = $this->_invitationData->getInvitationUrl($this);

        $templateVars = [
            'message' => $this->getMessage(),
            'store' => $store,
            'store_name' => $store->getGroup()->getName(),
            'inviter_name' => $this->getInviter() ? $this->getInviter()->getName() : null,
        ];

        foreach ($urls as $name => $url) {
            $templateVars[$name] = $url;
        }

        $this->_transportBuilder->setTemplateIdentifier(
            $templateIdentifier
        )->setTemplateOptions(
            ['area' => \Magento\Framework\App\Area::AREA_FRONTEND, 'store' => $this->getStoreId()]
        )->setTemplateVars(
            $templateVars
        )->setFrom(
            $from
        )->addTo(
            $this->getEmail()
        );
        $transport = $this->_transportBuilder->getTransport();
        try {
            $transport->sendMessage();
        } catch (\Magento\Framework\Exception\MailException $e) {
            return false;
        }
        $this->setStatus(\Magento\Invitation\Model\Invitation\Status::STATUS_SENT)->setUpdateDate(true)->save();
        return true;
    }

    public function accept($websiteId , $referralId)
    {
        $objectManager = ObjectManager::getInstance();
        $coreHelper = $objectManager->get('\FiloBlu\Referrals\Helper\Core');

        $couponInvitationType = $coreHelper->getSysConfigValue($coreHelper::COUPON_FIELD_INVITATION_TYPE);

        $this->makeSureCanBeAccepted($websiteId);

        $this->getResource()->beginTransaction();
        try {
            $this->setReferralId(
                $referralId
            )->setStatus(
                \Magento\Invitation\Model\Invitation\Status::STATUS_ACCEPTED
            )->setSignupDate(
                $this->dateTime->formatDate(time())
            )->save();
            $inviterId = $this->getCustomerId();

            if ($inviterId) {
                if ($inviterId === $referralId && $this->hasData('subscriber_id')) {
                    $this->getResource()->trackNlSubscriptionReferral($inviterId, $this->getSubscriberId());
                } else {
                    $this->getResource()->trackReferral($inviterId, $referralId);
                }
            }

            if ($couponInvitationType != 'newsletter') {
                $this->assignInvitationDataToCustomer($referralId);
            }

            $this->getResource()->commit();
        } catch (\Exception $e) {
            $this->getResource()->rollBack();
            throw $e;
        }

        return $this;
    }
}